@@ -16,7 +16,7 @@
 ; along with this program; if not, write to the Free Software Foundation,
 ; Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ;
-; $Id: oracle.lisp,v 1.21 2005/05/20 20:03:51 haible Exp $
+; $Id: oracle.lisp,v 1.24 2005/11/17 23:09:56 sds Exp $
 
 (defpackage "ORACLE"
   (:documentation
@@ -31,6 +31,8 @@
 (in-package "ORACLE")
 (pushnew :oracle *features*)
 
+(setf (documentation (find-package "ORACLE") 'sys::impnotes) "oracle")
+
 ; Use "C" as foreign language
 (default-foreign-language :stdc)
 
@@ -98,14 +100,16 @@ to-sqlval to-string update-row valid-symbol
 
 ; CONNECT
 
-(defun connect (user password server &optional schema (auto-commit t) (prefetch-buffer-bytes 0) (long-len -1) truncate-ok)
-"(ORACLE::CONNECT user password server &optional schema auto-commit prefetch-buffer-bytes long-len truncate-ok)
+(defun connect (user password server &optional schema (auto-commit t) (prefetch-buffer-bytes 0) (long-len -1) truncate-ok connid)
+"(ORACLE::CONNECT user password server &optional schema auto-commit prefetch-buffer-bytes long-len truncate-ok connid)
 
 Connect to an Oracle database.  All subsequent operations will affect
 this database.  A single program can access several different
 databases by repeated calls to CONNECT.  Connections are cached: if
-you call CONNECT again with the same arguments, the actual Oracle
-connection will be re-used.  CONNECT may not be called inside the
+you call CONNECT again with the same user, schema and server, the
+actual Oracle connection will be re-used.  This can be overriden by
+using the CONNID parameter to identify multiple connections to the
+same user, schema and server.  CONNECT may not be called inside the
 WITH-TRANSACTION macro.
 
 Required arguments:
@@ -131,6 +135,8 @@ Optional arguments:
                  will disable long fetching entirely.  If NIL or negative, defaults to
                  500k bytes.
     truncate-ok  If set, allow truncation of LONG columns to long-len (default: NIL).
+    connid       String assigning an ID making the connection unique among other
+                 connections to with the same (user schema sever)
 
 Returns: T if a cached connection was re-used (NIL if a new connection
          was created and cached).
@@ -147,7 +153,7 @@ Returns: T if a cached connection was re-used (NIL if a new connection
       (setf *oracle-connection-cache* (make-hash-table :test #'equal)))
 
   ; Construct key for connection cache
-  (let* ((hkey (connection-key user schema server))
+  (let* ((hkey (connection-key user schema server connid))
          (conn (gethash hkey *oracle-connection-cache*))
          (result t))
 
@@ -763,13 +769,22 @@ Argument: none
 
 ; Convert Oracle type based on sqlcol data type.  Oracle numerics are converted
 ; to the appropriate internal Lisp type using READ-FROM-STRING.  NULL is retained
-; as Lisp NIL, and strings and dates are left as Lisp string.
+; as Lisp NIL, BLOB and BFILE are converted to array of bytes,
+; and strings and dates are left as Lisp string.
 (defun convert-type (val sc)
   (let ((dtype (sqlcol-type sc)))
     (cond ((null val) nil)
+		  ((find dtype '("BLOB" "BFILE") :test #'equal)
+		   (hex-to-byte-array val))
           ((find dtype '("NUMBER" "INTEGER" "FLOAT") :test #'equal)
-           (read-from-string val))
-          ((find dtype '("VARCHAR" "DATE" "CHAR" "VARCHAR2" "LONG" "RAW" "LONG RAW" "BLOB" "CLOB" "BFILE" "ROWID DESC") :test #'equal)
+		   (let ((old-default-format *read-default-float-format*))
+			 ;; Adjust default float format, read, then restore old value
+			 (unwind-protect
+				 (progn
+				   (setf *read-default-float-format* 'double-float)
+				   (read-from-string val))
+			   (setf *read-default-float-format* old-default-format))))
+          ((find dtype '("VARCHAR" "DATE" "CHAR" "VARCHAR2" "LONG" "RAW" "LONG RAW" "CLOB" "ROWID DESC") :test #'equal)
            val)
           (t (db-error (cat "Unsupported data type '" dtype "'"))))))
 
@@ -821,10 +836,13 @@ Argument: none
 
 ; CONNECTION-KEY
 ; Construct key suitable for use in hash table keyed on
-; unique triple of (user, schema, server)
-(defun connection-key (user schema server)
+; unique triple of (user, schema, server) and optional connection ID
+(defun connection-key (user schema server &optional connid)
   ; Use ~-delimited string - pretty disgusting, eh?
-  (cat (string-upcase user) "~" (string-upcase schema) "~" (string-upcase server)))
+  (let ((result (cat (string-upcase user) "~" (string-upcase schema) "~" (string-upcase server))))
+	(when connid
+	  (setf result (cat result "~" connid)))
+	result))
 
 ; PAIRS-TO-HASH
 ; Convert a list of pairs ((key1 val1) (key2 val2) ...) to hash, enforcing key uniqueness
@@ -1051,6 +1069,33 @@ Argument: none
 (defun c-truth (x)
   (if (lisp-truth x) 1 0))
 
+;; HEX-VALUE -- Get integer value of single upper-case hex digit
+(defun hex-value (h)
+  (cond ((and (char>= h #\A) (char<= h #\F))
+         (+ 10 (- (char-code h) (char-code #\A))))
+        ((and (char>= h #\a) (char<= h #\f))
+         (+ 10 (- (char-code h) (char-code #\a))))
+        ((and (char>= h #\0) (char<= h #\9))
+         (- (char-code h) (char-code #\0)))
+        (t (error "Invalid hex digit"))))
+
+;; HEX-BYTE-VALUE -- Get byte value of pair of hex digits
+(defun hex-byte-value (hh)
+  (+ (* 16 (hex-value (elt hh 0)))
+     (hex-value (elt hh 1))))
+
+;; HEX-TO-BYTE-ARRAY -- Convert hex string to byte array
+(defun hex-to-byte-array (h)
+  (let* ((size (/ (length h) 2))
+         (result (make-array size :element-type '(unsigned-byte 8))))
+    (loop
+     for i from 0 to (1- size) do
+     (let ((offset (* 2 i)))
+       (setf (elt result i)
+             (coerce (hex-byte-value (subseq h offset (+ 2 offset)))
+                     '(unsigned-byte 8)))))
+    result))
+
 ; FLATTEN
 ; Flatten list (lifted from Paul Graham)
 (defun flatten (x)
