@@ -169,7 +169,7 @@ the actual object #<MACRO expander> for the FENV.
            (return-from nil (analyze-aux (cdr listr) name)))
           ((or (eq next '&ENVIRONMENT) (eq next '&WHOLE) (eq next '&REST)
                (eq next '&OPTIONAL) (eq next '&BODY) (eq next '&KEY))
-           (cerror (TEXT "It will be ignored.")
+           (cerror #1=(TEXT "It will be ignored.")
                    (TEXT "The lambda list of macro ~S contains a badly placed ~S.")
                    name next))
           (t
@@ -183,8 +183,7 @@ the actual object #<MACRO expander> for the FENV.
                         (cons `(,next (GETF ,restvar ',kw NIL)) %let-list))
                   (setq kwlist (cons kw kwlist)))
                  ((atom next)
-                  (cerror (TEXT "It will be ignored.")
-                          (TEXT "The lambda list of macro ~S contains the invalid element ~S")
+                  (cerror #1# (TEXT "The lambda list of macro ~S contains the invalid element ~S")
                           name next))
                  ((symbolp (car next))  ; (foo ...)
                   (setq kw (symbol-to-keyword (car next)))
@@ -325,7 +324,7 @@ the actual object #<MACRO expander> for the FENV.
                name listr)))
           ((eq item '&OPTIONAL)
            (if within-optional
-               (cerror (TEXT "It will be ignored.")
+               (cerror #2=(TEXT "It will be ignored.")
                        (TEXT "The lambda list of macro ~S contains a superfluous ~S.")
                        name item))
            (setq within-optional t))
@@ -337,12 +336,10 @@ the actual object #<MACRO expander> for the FENV.
            (setq %let-list (cons `(,g ,accessexp) %let-list))
            (return-from nil (analyze-key (cdr listr) g name)))
           ((eq item '&ALLOW-OTHER-KEYS)
-           (cerror (TEXT "It will be ignored.")
-                   (TEXT "The lambda list of macro ~S contains ~S before &KEY.")
+           (cerror #2# (TEXT "The lambda list of macro ~S contains ~S before &KEY.")
                    name item))
           ((eq item '&ENVIRONMENT)
-           (cerror (TEXT "It will be ignored.")
-                   (TEXT "The lambda list of macro ~S contains ~S which is illegal here.")
+           (cerror #2# (TEXT "The lambda list of macro ~S contains ~S which is illegal here.")
                    name item))
           ((eq item '&AUX)
            (return-from nil (analyze-aux (cdr listr) name)))
